# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::NetApp::Mgmt::V2020_06_01
  #
  # Microsoft NetApp Azure Resource Provider specification
  #
  class NetAppResource
    include MsRestAzure

    #
    # Creates and initializes a new instance of the NetAppResource class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [AzureNetAppFilesManagementClient] reference to the AzureNetAppFilesManagementClient
    attr_reader :client

    #
    # Check resource name availability
    #
    # Check if a resource name is available.
    #
    # @param location [String] The location
    # @param name [String] Resource name to verify.
    # @param type [CheckNameResourceTypes] Resource type used for verification.
    # Possible values include: 'Microsoft.NetApp/netAppAccounts',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes/snapshots'
    # @param resource_group [String] Resource group name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [CheckAvailabilityResponse] operation results.
    #
    def check_name_availability(location, name, type, resource_group, custom_headers:nil)
      response = check_name_availability_async(location, name, type, resource_group, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Check resource name availability
    #
    # Check if a resource name is available.
    #
    # @param location [String] The location
    # @param name [String] Resource name to verify.
    # @param type [CheckNameResourceTypes] Resource type used for verification.
    # Possible values include: 'Microsoft.NetApp/netAppAccounts',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes/snapshots'
    # @param resource_group [String] Resource group name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def check_name_availability_with_http_info(location, name, type, resource_group, custom_headers:nil)
      check_name_availability_async(location, name, type, resource_group, custom_headers:custom_headers).value!
    end

    #
    # Check resource name availability
    #
    # Check if a resource name is available.
    #
    # @param location [String] The location
    # @param name [String] Resource name to verify.
    # @param type [CheckNameResourceTypes] Resource type used for verification.
    # Possible values include: 'Microsoft.NetApp/netAppAccounts',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes/snapshots'
    # @param resource_group [String] Resource group name.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def check_name_availability_async(location, name, type, resource_group, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'location is nil' if location.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'name is nil' if name.nil?
      fail ArgumentError, 'type is nil' if type.nil?
      fail ArgumentError, 'resource_group is nil' if resource_group.nil?

      body = ResourceNameAvailabilityRequest.new
      unless name.nil? && type.nil? && resource_group.nil?
        body.name = name
        body.type = type
        body.resource_group = resource_group
      end

      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::NetApp::Mgmt::V2020_06_01::Models::ResourceNameAvailabilityRequest.mapper()
      request_content = @client.serialize(request_mapper,  body)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.NetApp/locations/{location}/checkNameAvailability'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'location' => location},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::NetApp::Mgmt::V2020_06_01::Models::CheckAvailabilityResponse.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Check file path availability
    #
    # Check if a file path is available.
    #
    # @param location [String] The location
    # @param name [String] Resource name to verify.
    # @param type [CheckNameResourceTypes] Resource type used for verification.
    # Possible values include: 'Microsoft.NetApp/netAppAccounts',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes/snapshots'
    # @param resource_group [String] Resource group name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [CheckAvailabilityResponse] operation results.
    #
    def check_file_path_availability(location, name, type, resource_group, custom_headers:nil)
      response = check_file_path_availability_async(location, name, type, resource_group, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Check file path availability
    #
    # Check if a file path is available.
    #
    # @param location [String] The location
    # @param name [String] Resource name to verify.
    # @param type [CheckNameResourceTypes] Resource type used for verification.
    # Possible values include: 'Microsoft.NetApp/netAppAccounts',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes/snapshots'
    # @param resource_group [String] Resource group name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def check_file_path_availability_with_http_info(location, name, type, resource_group, custom_headers:nil)
      check_file_path_availability_async(location, name, type, resource_group, custom_headers:custom_headers).value!
    end

    #
    # Check file path availability
    #
    # Check if a file path is available.
    #
    # @param location [String] The location
    # @param name [String] Resource name to verify.
    # @param type [CheckNameResourceTypes] Resource type used for verification.
    # Possible values include: 'Microsoft.NetApp/netAppAccounts',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes/snapshots'
    # @param resource_group [String] Resource group name.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def check_file_path_availability_async(location, name, type, resource_group, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'location is nil' if location.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'name is nil' if name.nil?
      fail ArgumentError, 'type is nil' if type.nil?
      fail ArgumentError, 'resource_group is nil' if resource_group.nil?

      body = ResourceNameAvailabilityRequest.new
      unless name.nil? && type.nil? && resource_group.nil?
        body.name = name
        body.type = type
        body.resource_group = resource_group
      end

      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::NetApp::Mgmt::V2020_06_01::Models::ResourceNameAvailabilityRequest.mapper()
      request_content = @client.serialize(request_mapper,  body)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.NetApp/locations/{location}/checkFilePathAvailability'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'location' => location},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::NetApp::Mgmt::V2020_06_01::Models::CheckAvailabilityResponse.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Check quota availability
    #
    # Check if a quota is available.
    #
    # @param location [String] The location
    # @param name [String] Name of the resource to verify.
    # @param type [CheckQuotaNameResourceTypes] Resource type used for
    # verification. Possible values include: 'Microsoft.NetApp/netAppAccounts',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes/snapshots'
    # @param resource_group [String] Resource group name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [CheckAvailabilityResponse] operation results.
    #
    def check_quota_availability(location, name, type, resource_group, custom_headers:nil)
      response = check_quota_availability_async(location, name, type, resource_group, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Check quota availability
    #
    # Check if a quota is available.
    #
    # @param location [String] The location
    # @param name [String] Name of the resource to verify.
    # @param type [CheckQuotaNameResourceTypes] Resource type used for
    # verification. Possible values include: 'Microsoft.NetApp/netAppAccounts',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes/snapshots'
    # @param resource_group [String] Resource group name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def check_quota_availability_with_http_info(location, name, type, resource_group, custom_headers:nil)
      check_quota_availability_async(location, name, type, resource_group, custom_headers:custom_headers).value!
    end

    #
    # Check quota availability
    #
    # Check if a quota is available.
    #
    # @param location [String] The location
    # @param name [String] Name of the resource to verify.
    # @param type [CheckQuotaNameResourceTypes] Resource type used for
    # verification. Possible values include: 'Microsoft.NetApp/netAppAccounts',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes',
    # 'Microsoft.NetApp/netAppAccounts/capacityPools/volumes/snapshots'
    # @param resource_group [String] Resource group name.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def check_quota_availability_async(location, name, type, resource_group, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'location is nil' if location.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'name is nil' if name.nil?
      fail ArgumentError, 'type is nil' if type.nil?
      fail ArgumentError, 'resource_group is nil' if resource_group.nil?

      body = QuotaAvailabilityRequest.new
      unless name.nil? && type.nil? && resource_group.nil?
        body.name = name
        body.type = type
        body.resource_group = resource_group
      end

      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::NetApp::Mgmt::V2020_06_01::Models::QuotaAvailabilityRequest.mapper()
      request_content = @client.serialize(request_mapper,  body)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.NetApp/locations/{location}/checkQuotaAvailability'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'location' => location},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::NetApp::Mgmt::V2020_06_01::Models::CheckAvailabilityResponse.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

  end
end
